@@ -6,6 +6,7 @@ from dataclasses import dataclass, is_dataclass, fields
 from enum import Enum
 from functools import partial
 from pathlib import Path
+from types import UnionType
 from typing import Tuple, Dict, Union, Type, TypeVar, get_type_hints
 
 import numpy as np
@@ -14,6 +15,8 @@ import numpy as np
 # e.g. var_a: str = field(metadata=EXCLUDE)
 # add other meta data through dictionary union operator:
 # e.g. var_a: str = field(metadata=EXCLUDE|{'other_metakey': some_val})
+from strenum import StrEnum
+
 EXCLUDE_KEY = 'exclude'
 EXCLUDE = {EXCLUDE_KEY: True}
 
@@ -54,13 +57,16 @@ def if_key_exists_increment_id(dic: dict, name: str, id: int = 1):
 
 built_in_types = [int, float, str, bool, complex, type(None)]
 built_in_types_names = [t.__name__ for t in built_in_types]
+built_in_types_names_class = {name: _class for name, _class in zip(built_in_types_names, built_in_types)}
 
 
 def specify_type_for_special_cases(json_object, _field, value):
     # example cases:
     # if field is of type Union[int, str] the type at reconstruction time would be ambiguous and must be stored.
+    # if field is of type int|str the type at reconstruction time would be ambiguous and must be stored.
     # if field is of type Base and value is of type SubClass(Base), the path to the subclass must be stored.
     conditions = [hasattr(_field.type, '__origin__') and _field.type.__origin__ == Union,
+                  isinstance(_field.type, UnionType),
                   # required for future imports: and type(value).__name__ != _field.type
                   type(value) != _field.type and value is not None and type(value).__name__ != _field.type]
     if any(conditions):
@@ -73,6 +79,9 @@ def specify_type_for_special_cases(json_object, _field, value):
         elif type(value) in built_in_types:
             json_object[_field.name] = {'type': str(type(value).__name__),
                                         'value': json_object[_field.name]}
+        elif isinstance(value, StrEnum):  # e.g. if type is specified with str|SomeStrEnum
+            json_object[_field.name] = {'type': f'{type(value).__module__}|{type(value).__name__}',
+                                        'value': json_object[_field.name]}
 
 
 def do_preserve_field(field_, explicit: tuple[str]):
@@ -151,6 +160,9 @@ def dataclass_to_dicts(instance,
 
 def _get_cls_from_type_str(type_str, **kwargs):
     # https://stackoverflow.com/questions/4821104/dynamic-instantiation-from-string-name-of-a-class-in-dynamically-imported-module
+    # if type_str in built_in_types_names:
+    #     return built_in_types_names_class[type_str]
+    # else:
     module, cls_name = type_str.split('|')
     module_ = importlib.import_module(module)
     class_ = getattr(module_, cls_name)
@@ -162,9 +174,34 @@ def _create_instance_from_type_str(type_str, **kwargs):
     return class_(**kwargs)
 
 
+def deal_with_creation_for_union_type(data_dict):
+    if data_dict is not None and 'type' in data_dict:
+        if data_dict['type'] in built_in_types_names:
+            return create_instance_from_data_dict(built_in_types_names_class[data_dict['type']],
+                                                  data_dict['value'])
+        else:
+            type_instance = _get_cls_from_type_str(data_dict['type'])
+            if issubclass(type_instance, Enum):
+                return create_instance_from_data_dict(type_instance,
+                                                      data_dict['value'])
+            else:
+                type_instance = _get_cls_from_type_str(data_dict['type'])
+                return create_instance_from_data_dict(type_instance, data_dict)
+    else:
+        raise NotImplementedError()
+
+
 def create_instance_from_data_dict(type_instance,
                                    data_dict):
-    if hasattr(type_instance, '__origin__'):
+    if type_instance in [np.ndarray, np.array]:
+        return data_dict
+    elif type_instance in built_in_types:
+        return data_dict
+    # above are cases where data_dict is an elementary type, where no further recursion is required
+    # below are cases where data_dict encodes a container type like dataclasses, Enums, tuple, list, Union
+    elif isinstance(type_instance, type(Enum)):
+        return type_instance[data_dict]
+    elif hasattr(type_instance, '__origin__'):
         # https://stackoverflow.com/questions/48572831/how-to-access-the-type-arguments-of-typing-generic
         if type_instance.__origin__ == tuple:
             return tuple([create_instance_from_data_dict(type_instance.__args__[0], item)
@@ -176,18 +213,9 @@ def create_instance_from_data_dict(type_instance,
             return {key: create_instance_from_data_dict(type_instance.__args__[0], item)
                     for key, item in data_dict.items()}
         if type_instance.__origin__ == Union:
-            for arg in type_instance.__args__:
-                if data_dict['type'] in built_in_types_names:
-                    return create_instance_from_data_dict(arg, data_dict['value'])
-                elif data_dict['type'] in str(f'{arg.__module__}|{arg.__name__}'):
-                    return create_instance_from_data_dict(arg, data_dict)
-            raise ValueError('specific type not found in given Union type')
-    elif type_instance in [np.ndarray, np.array]:
-        return data_dict
-    elif type_instance in built_in_types:
-        return data_dict
-    elif isinstance(type_instance, type(Enum)):
-        return type_instance[data_dict]
+            return deal_with_creation_for_union_type(data_dict)
+    elif isinstance(type_instance, UnionType):
+        return deal_with_creation_for_union_type(data_dict)
     else:
         # in case of ambiguities in the type hint, the type information will be stored inside of the json file
         # the loaded type information replaces the one from the class definition.
@@ -375,6 +403,16 @@ def _replace_not_excluded_fields(old, new):
         if is_dataclass(old.__getattribute__(key)):
             if not val.metadata.get(EXCLUDE_KEY, False):
                 _replace_not_excluded_fields(old.__getattribute__(key), new.__getattribute__(key))
+        elif old.__getattribute__(key) is None:
+            old.__setattr__(key, new.__getattribute__(key))
+        elif hasattr(val.type, '__origin__') and val.type.__origin__ == list:
+            list_old, list_new = old.__getattribute__(key), new.__getattribute__(key)
+            # if list sizes do not agree, take the new list
+            if len(list_old) != len(list_new) or (len(list_new) > 0 and not is_dataclass(list_new[0])):
+                old.__setattr__(key, list_new)
+            else:
+                for _old, _new in zip(list_old, list_new):
+                    _replace_not_excluded_fields(_old, _new)
         else:
             if not val.metadata.get(EXCLUDE_KEY, False):
                 old.__setattr__(key, new.__getattribute__(key))
